@@ -18,11 +18,8 @@
 #include <stdio.h>
 #include <libgnomeui/gnome-canvas.h>
 #include <gtk/gtksignal.h>
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
 
 #include "gal/util/e-util.h"
-#include "gal/util/e-xml-utils.h"
 #include "gal/widgets/e-canvas.h"
 #include "gal/widgets/e-canvas-vbox.h"
 #include "e-table.h"
@@ -32,6 +29,9 @@
 #include "e-table-group.h"
 #include "e-table-group-leaf.h"
 #include "e-table-click-to-add.h"
+#include "e-table-specification.h"
+#include "e-table-state.h"
+#include "e-table-column-specification.h"
 
 #define COLUMN_HEADER_HEIGHT 16
 
@@ -61,11 +61,8 @@ enum {
 
 enum {
 	ARG_0,
-	ARG_TABLE_DRAW_GRID,
 	ARG_TABLE_DRAW_FOCUS,
-	ARG_CURSOR_MODE,
 	ARG_LENGTH_THRESHOLD,
-	ARG_CLICK_TO_ADD_MESSAGE,
 };
 
 static gint et_signals [LAST_SIGNAL] = { 0, };
@@ -147,6 +144,8 @@ et_destroy (GtkObject *object)
 	gtk_object_unref (GTK_OBJECT (et->sort_info));
 	gtk_object_unref (GTK_OBJECT (et->sorter));
 	gtk_object_unref (GTK_OBJECT (et->selection));
+	if (et->spec)
+		gtk_object_unref (GTK_OBJECT (et->spec));
 
 	if (et->header_canvas != NULL)
 		gtk_widget_destroy (GTK_WIDGET (et->header_canvas));
@@ -196,6 +195,7 @@ e_table_init (GtkObject *object)
 	e_table->sorter = NULL;
 	e_table->selection = e_table_selection_model_new();
 	e_table->cursor_loc = E_TABLE_CURSOR_LOC_NONE;
+	e_table->spec = NULL;
 }
 
 static void
@@ -238,6 +238,7 @@ e_table_setup_header (ETable *e_table)
 		"full_header", e_table->full_header,
 		"sort_info", e_table->sort_info,
 		"dnd_code", "(unset)",
+		"table", e_table,
 		NULL);
 
 	gtk_signal_connect (
@@ -252,18 +253,25 @@ static gboolean
 table_canvas_reflow_idle (ETable *e_table)
 {
 	gdouble height, width;
+	gdouble item_height;
 	GtkAllocation *alloc = &(GTK_WIDGET (e_table->table_canvas)->allocation);
 
 	gtk_object_get (GTK_OBJECT (e_table->canvas_vbox),
 			"height", &height,
 			"width", &width,
 			NULL);
+	item_height = height;
 	height = MAX ((int)height, alloc->height);
 	width = MAX((int)width, alloc->width);
 	/* I have no idea why this needs to be -1, but it works. */
 	gnome_canvas_set_scroll_region (
 		GNOME_CANVAS (e_table->table_canvas),
 		0, 0, width - 1, height - 1);
+	gtk_object_set (GTK_OBJECT (e_table->white_item),
+			"y1", item_height + 1,
+			"x2", width,
+			"y2", height,
+			NULL);
 	e_table->reflow_idle_id = 0;
 	return FALSE;
 }
@@ -273,7 +281,15 @@ table_canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc,
 			    ETable *e_table)
 {
 	gdouble width;
+	gdouble height;
+	gdouble item_height;
+
 	width = alloc->width;
+	gtk_object_get (GTK_OBJECT (e_table->canvas_vbox),
+			"height", &height,
+			NULL);
+	item_height = height;
+	height = MAX ((int)height, alloc->height);
 
 	gtk_object_set (GTK_OBJECT (e_table->canvas_vbox),
 			"width", width,
@@ -281,6 +297,11 @@ table_canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc,
 	gtk_object_set (GTK_OBJECT (e_table->header),
 			"width", width,
 			NULL);
+	gtk_object_set (GTK_OBJECT (e_table->white_item),
+			"y1", item_height + 1,
+			"x2", width,
+			"y2", height,
+			NULL);
 	if (e_table->reflow_idle_id)
 		g_source_remove(e_table->reflow_idle_id);
 	table_canvas_reflow_idle(e_table);
@@ -429,6 +450,9 @@ changed_idle (gpointer data)
 		gtk_object_set (GTK_OBJECT (et->canvas_vbox),
 				"width", (double) GTK_WIDGET (et->table_canvas)->allocation.width,
 				NULL);
+
+		if (GTK_WIDGET_REALIZED(et->table_canvas))
+			table_canvas_size_allocate (GTK_WIDGET(et->table_canvas), &GTK_WIDGET(et->table_canvas)->allocation, et);
 	}
 
 	et->need_rebuild = 0;
@@ -483,6 +507,14 @@ et_table_row_deleted (ETableModel *table_model, int row, ETable *et)
 	}
 }
 
+static void
+et_canvas_realize (GtkWidget *canvas, ETable *e_table)
+{
+	gnome_canvas_item_set(e_table->white_item,
+			      "fill_color_gdk", &GTK_WIDGET(e_table->table_canvas)->style->base[GTK_STATE_NORMAL],
+			      NULL);
+}
+
 static void
 e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *header,
 		     ETableModel *model)
@@ -491,6 +523,12 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 	gtk_signal_connect (
 		GTK_OBJECT (e_table->table_canvas), "size_allocate",
 		GTK_SIGNAL_FUNC (table_canvas_size_allocate), e_table);
+	gtk_signal_connect (
+		GTK_OBJECT (e_table->table_canvas), "focus_in_event",
+		GTK_SIGNAL_FUNC (gtk_widget_queue_draw), e_table);
+	gtk_signal_connect (
+		GTK_OBJECT (e_table->table_canvas), "focus_out_event",
+		GTK_SIGNAL_FUNC (gtk_widget_queue_draw), e_table);
 
 	gtk_signal_connect (
 		GTK_OBJECT (e_table), "drag_begin",
@@ -523,6 +561,16 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 	gtk_widget_show (GTK_WIDGET (e_table->table_canvas));
 
 
+	e_table->white_item = gnome_canvas_item_new(gnome_canvas_root(e_table->table_canvas),
+						    gnome_canvas_rect_get_type(),
+						    "x1", (double) 0,
+						    "y1", (double) 0,
+						    "x2", (double) 100,
+						    "y2", (double) 100,
+						    "fill_color_gdk", &GTK_WIDGET(e_table->table_canvas)->style->base[GTK_STATE_NORMAL],
+						    NULL);
+	gtk_signal_connect(GTK_OBJECT(e_table->table_canvas), "realize",
+			   GTK_SIGNAL_FUNC(et_canvas_realize), e_table);
 	e_table->canvas_vbox = gnome_canvas_item_new(gnome_canvas_root(e_table->table_canvas),
 						     e_canvas_vbox_get_type(),
 						     "spacing", 10.0,
@@ -594,26 +642,75 @@ e_table_fill_table (ETable *e_table, ETableModel *model)
 	e_table_group_add_all (e_table->group);
 }
 
+static ETableCol *
+et_col_spec_to_col (ETable *e_table, ETableColumnSpecification *col_spec, ETableExtras *ete)
+{
+	ETableCol *col = NULL;
+	ECell *cell;
+	GCompareFunc compare;
+
+	cell = e_table_extras_get_cell(ete, col_spec->cell);
+	compare = e_table_extras_get_compare(ete, col_spec->compare);
+
+	if (cell && compare) {
+		if (col_spec->pixbuf && *col_spec->pixbuf) {
+			GdkPixbuf *pixbuf;
+			
+			pixbuf = e_table_extras_get_pixbuf(ete, col_spec->pixbuf);
+			if (pixbuf) {
+				col = e_table_col_new_with_pixbuf (col_spec->model_col, pixbuf,
+								   col_spec->expansion, col_spec->minimum_width,
+								   cell, compare, col_spec->resizable);
+			}
+		} 
+		if (col == NULL && col_spec->title_ && *col_spec->title_) {
+			col = e_table_col_new (col_spec->model_col, col_spec->title_,
+					       col_spec->expansion, col_spec->minimum_width,
+					       cell, compare, col_spec->resizable);
+		}
+	}
+	return col;
+}
+
 static ETableHeader *
-et_xml_to_header (ETable *e_table, ETableHeader *full_header, xmlNode *xmlColumns)
+et_spec_to_full_header (ETable *e_table, ETableSpecification *spec, ETableExtras *ete)
 {
 	ETableHeader *nh;
-	xmlNode *column;
-	const int max_cols = e_table_header_count (full_header);
+	int column;
 
 	g_return_val_if_fail (e_table, NULL);
-	g_return_val_if_fail (full_header, NULL);
-	g_return_val_if_fail (xmlColumns, NULL);
+	g_return_val_if_fail (spec, NULL);
+	g_return_val_if_fail (ete, NULL);
 	
 	nh = e_table_header_new ();
 
-	for (column = xmlColumns->childs; column; column = column->next) {
-		gchar *content;
+	for (column = 0; spec->columns[column]; column++) {
+		ETableCol *col = et_col_spec_to_col(e_table, spec->columns[column], ete);
+
+		if (col)
+			e_table_header_add_column (nh, col, -1);
+	}
+
+	return nh;
+}
+
+static ETableHeader *
+et_state_to_header (ETable *e_table, ETableHeader *full_header, ETableState *state)
+{
+	ETableHeader *nh;
+	const int max_cols = e_table_header_count (full_header);
+	int column;
+
+	g_return_val_if_fail (e_table, NULL);
+	g_return_val_if_fail (full_header, NULL);
+	g_return_val_if_fail (state, NULL);
+	
+	nh = e_table_header_new ();
+
+	for (column = 0; column < state->col_count; column++) {
 		int col;
 
-		content = xmlNodeListGetString (column->doc, column->childs, 1);
-		col = atoi (content);
-		xmlFree (content);
+		col = state->columns[column];
 
 		if (col >= max_cols)
 			continue;
@@ -625,61 +722,138 @@ et_xml_to_header (ETable *e_table, ETableHeader *full_header, xmlNode *xmlColumn
 }
 
 static void
-et_grouping_xml_to_sort_info (ETable *table, xmlNode *grouping)
+et_set_state(ETable *e_table, ETableState *state)
 {
-	int i;
+	if (e_table->header)
+		gtk_object_unref(GTK_OBJECT(e_table->header));
+	e_table->header = et_state_to_header (e_table, e_table->full_header, state);
+	if (e_table->header)
+		gtk_object_ref(GTK_OBJECT(e_table->header));
 
-	g_return_if_fail (table!=NULL);
-	g_return_if_fail (grouping!=NULL);	
-	
-	table->sort_info = e_table_sort_info_new ();
-	
-	gtk_object_ref (GTK_OBJECT (table->sort_info));
-	gtk_object_sink (GTK_OBJECT (table->sort_info));
+	if (e_table->sort_info)
+		gtk_object_unref(GTK_OBJECT(e_table->sort_info));
+	e_table->sort_info = state->sort_info;
+	if (e_table->sort_info)
+		gtk_object_ref(GTK_OBJECT(e_table->sort_info));
 
-	i = 0;
-	for (grouping = grouping->childs; grouping && !strcmp (grouping->name, "group"); grouping = grouping->childs) {
-		ETableSortColumn column;
-		column.column = e_xml_get_integer_prop_by_name (grouping, "column");
-		column.ascending = e_xml_get_integer_prop_by_name (grouping, "ascending");
-		e_table_sort_info_grouping_set_nth(table->sort_info, i++, column);
-	}
-	i = 0;
-	for (; grouping && !strcmp (grouping->name, "leaf"); grouping = grouping->childs) {
-		ETableSortColumn column;
-		column.column = e_xml_get_integer_prop_by_name (grouping, "column");
-		column.ascending = e_xml_get_integer_prop_by_name (grouping, "ascending");
-		e_table_sort_info_sorting_set_nth(table->sort_info, i++, column);
-	}
+	if (e_table->header_item)
+		gtk_object_set(GTK_OBJECT(e_table->header_item),
+			       "ETableHeader", e_table->header,
+			       "sort_info", e_table->sort_info,
+			       NULL);
+	if (e_table->click_to_add)
+		gtk_object_set(GTK_OBJECT(e_table->click_to_add),
+			       "header", e_table->header,
+			       NULL);
 
-	table->group_info_change_id = 
-		gtk_signal_connect (GTK_OBJECT (table->sort_info), "group_info_changed", 
-				    GTK_SIGNAL_FUNC (sort_info_changed), table);
+	e_table->need_rebuild = TRUE;
+	if (!e_table->rebuild_idle_id)
+		e_table->rebuild_idle_id = g_idle_add_full (20, changed_idle, e_table, NULL);
 }
 
-static ETable *
-et_real_construct (ETable *e_table, ETableHeader *full_header, ETableModel *etm,
-		   xmlDoc *xmlSpec)
+void            e_table_set_state                 (ETable               *e_table,
+						   const gchar          *state_str)
 {
-	xmlNode *xmlRoot;
-	xmlNode *xmlColumns;
-	xmlNode *xmlGrouping;
-	gboolean no_header;
-	int row = 0;
+	ETableState *state;
 
-	xmlRoot = xmlDocGetRootElement (xmlSpec);
-	xmlColumns = e_xml_get_child_by_name (xmlRoot, "columns-shown");
-	xmlGrouping = e_xml_get_child_by_name (xmlRoot, "grouping");
+	g_return_if_fail(e_table != NULL);
+	g_return_if_fail(E_IS_TABLE(e_table));
+	g_return_if_fail(state_str != NULL);
 
-	if ((xmlColumns == NULL) || (xmlGrouping == NULL))
-		return NULL;
+	state = e_table_state_new();
+	e_table_state_load_from_string(state, state_str);
 
-	no_header = e_xml_get_integer_prop_by_name(xmlRoot, "no-header");
-	e_table->use_click_to_add = e_xml_get_integer_prop_by_name(xmlRoot, "click-to-add");
+	if (state->col_count > 0)
+		et_set_state(e_table, state);
+
+	gtk_object_sink(GTK_OBJECT(state));
+}
+
+void            e_table_load_state                (ETable               *e_table,
+						   const gchar          *filename)
+{
+	ETableState *state;
+
+	g_return_if_fail(e_table != NULL);
+	g_return_if_fail(E_IS_TABLE(e_table));
+	g_return_if_fail(filename != NULL);
+
+	state = e_table_state_new();
+	e_table_state_load_from_file(state, filename);
+
+	if (state->col_count > 0)
+		et_set_state(e_table, state);
+
+	gtk_object_sink(GTK_OBJECT(state));
+}
+
+ETableState *
+e_table_get_state_object (ETable *e_table)
+{
+	ETableState *state;
+	int full_col_count;
+	int i, j;
+
+	state = e_table_state_new();
+	state->sort_info = e_table->sort_info;
+	gtk_object_ref(GTK_OBJECT(state->sort_info));
 
 	
-	e_table->full_header = full_header;
-	gtk_object_ref (GTK_OBJECT (full_header));
+	state->col_count = e_table_header_count (e_table->header);
+	full_col_count = e_table_header_count (e_table->full_header);
+	state->columns = g_new(int, state->col_count);
+	for (i = 0; i < state->col_count; i++) {
+		int col_idx = e_table_header_index(e_table->header, i);
+		state->columns[i] = -1;
+		for (j = 0; j < full_col_count; j++) {
+			if (col_idx == e_table_header_index(e_table->full_header, j)) {
+				state->columns[i] = j;
+				break;
+			}
+		}
+	}
+
+	return state;
+}
+
+gchar          *e_table_get_state                 (ETable               *e_table)
+{
+	ETableState *state;
+	gchar *string;
+
+	state = e_table_get_state_object(e_table);
+	string = e_table_state_save_to_string(state);
+	gtk_object_sink(GTK_OBJECT(state));
+	return string;
+}
+
+void            e_table_save_state                (ETable               *e_table,
+						   const gchar          *filename)
+{
+	ETableState *state;
+
+	state = e_table_get_state_object(e_table);
+	e_table_state_save_to_file(state, filename);
+	gtk_object_sink(GTK_OBJECT(state));
+}
+
+
+static ETable *
+et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
+		   ETableSpecification *specification, ETableState *state)
+{
+	int row = 0;
+
+	if (ete)
+		gtk_object_ref(GTK_OBJECT(ete));
+	else
+		ete = e_table_extras_new();
+
+	e_table->use_click_to_add = specification->click_to_add;
+	e_table->click_to_add_message = g_strdup(specification->click_to_add_message_);
+	e_table->draw_grid = specification->draw_grid;
+	e_table->cursor_mode = specification->cursor_mode;
+	e_table->full_header = et_spec_to_full_header(e_table, specification, ete);
 
 	e_table->model = etm;
 	gtk_object_ref (GTK_OBJECT (etm));
@@ -687,8 +861,13 @@ et_real_construct (ETable *e_table, ETableHeader *full_header, ETableModel *etm,
 	gtk_widget_push_visual (gdk_rgb_get_visual ());
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 
-	e_table->header = et_xml_to_header (e_table, full_header, xmlColumns);
-	et_grouping_xml_to_sort_info (e_table, xmlGrouping);
+	e_table->header = et_state_to_header (e_table, e_table->full_header, state);
+	e_table->sort_info = state->sort_info;
+
+	e_table->group_info_change_id =
+		gtk_signal_connect (GTK_OBJECT (e_table->sort_info), "group_info_changed",
+				    GTK_SIGNAL_FUNC (sort_info_changed), e_table);
+
 	
 	gtk_object_set(GTK_OBJECT(e_table->header),
 		       "sort_info", e_table->sort_info,
@@ -701,16 +880,16 @@ et_real_construct (ETable *e_table, ETableHeader *full_header, ETableModel *etm,
 			"sorter", e_table->sorter,
 			NULL);
 
-	if (!no_header) {
+	if (!specification->no_headers) {
 		e_table_setup_header (e_table);
 	}
-	e_table_setup_table (e_table, full_header, e_table->header, etm);
+	e_table_setup_table (e_table, e_table->full_header, e_table->header, etm);
 	e_table_fill_table (e_table, etm);
 
 	gtk_layout_get_vadjustment (GTK_LAYOUT (e_table->table_canvas))->step_increment = 20;
 	gtk_adjustment_changed(gtk_layout_get_vadjustment (GTK_LAYOUT (e_table->table_canvas)));
 
-	if (!no_header) {
+	if (!specification->no_headers) {
 		/*
 		 * The header
 		 */
@@ -729,91 +908,128 @@ et_real_construct (ETable *e_table, ETableHeader *full_header, ETableModel *etm,
 	gtk_widget_pop_colormap ();
 	gtk_widget_pop_visual ();
 
-	return e_table;
-}
-
-ETable *
-e_table_construct (ETable *e_table, ETableHeader *full_header,
-		   ETableModel *etm, const char *spec)
-{
-	xmlDoc *xmlSpec;
-	char *copy;
-
-	g_return_val_if_fail(e_table != NULL, NULL);
-	g_return_val_if_fail(E_IS_TABLE(e_table), NULL);
-	g_return_val_if_fail(full_header != NULL, NULL);
-	g_return_val_if_fail(E_IS_TABLE_HEADER(full_header), NULL);
-	g_return_val_if_fail(etm != NULL, NULL);
-	g_return_val_if_fail(E_IS_TABLE_MODEL(etm), NULL);
-	g_return_val_if_fail(spec != NULL, NULL);
-
-	copy = g_strdup (spec);
-
-	xmlSpec = xmlParseMemory (copy, strlen(copy));
-	e_table = et_real_construct (e_table, full_header, etm, xmlSpec);
-	xmlFreeDoc (xmlSpec);
-	g_free (copy);
+	gtk_object_unref(GTK_OBJECT(ete));
 
 	return e_table;
 }
 
 ETable *
-e_table_construct_from_spec_file (ETable *e_table, ETableHeader *full_header, ETableModel *etm,
-				  const char *filename)
+e_table_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
+		   const char *spec_str, const char *state_str)
 {
-	xmlDoc *xmlSpec;
+	ETableSpecification *specification;
+	ETableState *state;
 
 	g_return_val_if_fail(e_table != NULL, NULL);
 	g_return_val_if_fail(E_IS_TABLE(e_table), NULL);
-	g_return_val_if_fail(full_header != NULL, NULL);
-	g_return_val_if_fail(E_IS_TABLE_HEADER(full_header), NULL);
 	g_return_val_if_fail(etm != NULL, NULL);
 	g_return_val_if_fail(E_IS_TABLE_MODEL(etm), NULL);
-	g_return_val_if_fail(filename != NULL, NULL);
+	g_return_val_if_fail(ete == NULL || E_IS_TABLE_EXTRAS(ete), NULL);
+	g_return_val_if_fail(spec_str != NULL, NULL);
 
-	xmlSpec = xmlParseFile (filename);
-	e_table = et_real_construct (e_table, full_header, etm, xmlSpec);
-	xmlFreeDoc (xmlSpec);
+	specification = e_table_specification_new();
+	e_table_specification_load_from_string(specification, spec_str);
+	if (state_str) {
+		state = e_table_state_new();
+		e_table_state_load_from_string(state, state_str);
+		if (state->col_count <= 0) {
+			gtk_object_unref(GTK_OBJECT(state));
+			state = specification->state;
+			gtk_object_ref(GTK_OBJECT(state));
+		}
+	} else {
+		state = specification->state;
+		gtk_object_ref(GTK_OBJECT(state));
+	}
+
+	e_table = et_real_construct (e_table, etm, ete, specification, state);
+
+	e_table->spec = specification;
+	gtk_object_unref(GTK_OBJECT(state));
+
+	return e_table;
+}
+
+ETable *
+e_table_construct_from_spec_file (ETable *e_table, ETableModel *etm, ETableExtras *ete,
+				  const char *spec_fn, const char *state_fn)
+{
+	ETableSpecification *specification;
+	ETableState *state;
+
+	g_return_val_if_fail(e_table != NULL, NULL);
+	g_return_val_if_fail(E_IS_TABLE(e_table), NULL);
+	g_return_val_if_fail(etm != NULL, NULL);
+	g_return_val_if_fail(E_IS_TABLE_MODEL(etm), NULL);
+	g_return_val_if_fail(ete == NULL || E_IS_TABLE_EXTRAS(ete), NULL);
+	g_return_val_if_fail(spec_fn != NULL, NULL);
+
+	specification = e_table_specification_new();
+	if (!e_table_specification_load_from_file(specification, spec_fn)) {
+		gtk_object_unref(GTK_OBJECT(specification));
+		return NULL;
+	}
+
+	if (state_fn) {
+		state = e_table_state_new();
+		if (!e_table_state_load_from_file(state, state_fn)) {
+			gtk_object_unref(GTK_OBJECT(state));
+			state = specification->state;
+			gtk_object_ref(GTK_OBJECT(state));
+		}
+		if (state->col_count <= 0) {
+			gtk_object_unref(GTK_OBJECT(state));
+			state = specification->state;
+			gtk_object_ref(GTK_OBJECT(state));
+		}
+	} else {
+		state = specification->state;
+		gtk_object_ref(GTK_OBJECT(state));
+	}
+
+	e_table = et_real_construct (e_table, etm, ete, specification, state);
+
+	e_table->spec = specification;
+	gtk_object_unref(GTK_OBJECT(state));
 
 	return e_table;
 }
 
 GtkWidget *
-e_table_new (ETableHeader *full_header, ETableModel *etm, const char *spec)
+e_table_new (ETableModel *etm, ETableExtras *ete, const char *spec, const char *state)
 {
 	ETable *e_table;
 
-	g_return_val_if_fail(full_header != NULL, NULL);
-	g_return_val_if_fail(E_IS_TABLE_HEADER(full_header), NULL);
 	g_return_val_if_fail(etm != NULL, NULL);
 	g_return_val_if_fail(E_IS_TABLE_MODEL(etm), NULL);
+	g_return_val_if_fail(ete == NULL || E_IS_TABLE_EXTRAS(ete), NULL);
 	g_return_val_if_fail(spec != NULL, NULL);
 
 	e_table = gtk_type_new (e_table_get_type ());
 
-	e_table = e_table_construct (e_table, full_header, etm, spec);
+	e_table = e_table_construct (e_table, etm, ete, spec, state);
 		
 	return GTK_WIDGET (e_table);
 }
 
 GtkWidget *
-e_table_new_from_spec_file (ETableHeader *full_header, ETableModel *etm, const char *filename)
+e_table_new_from_spec_file (ETableModel *etm, ETableExtras *ete, const char *spec_fn, const char *state_fn)
 {
 	ETable *e_table;
 
-	g_return_val_if_fail(full_header != NULL, NULL);
-	g_return_val_if_fail(E_IS_TABLE_HEADER(full_header), NULL);
 	g_return_val_if_fail(etm != NULL, NULL);
 	g_return_val_if_fail(E_IS_TABLE_MODEL(etm), NULL);
-	g_return_val_if_fail(filename != NULL, NULL);
+	g_return_val_if_fail(ete == NULL || E_IS_TABLE_EXTRAS(ete), NULL);
+	g_return_val_if_fail(spec_fn != NULL, NULL);
 
 	e_table = gtk_type_new (e_table_get_type ());
 
-	e_table = e_table_construct_from_spec_file (e_table, full_header, etm, filename);
+	e_table = e_table_construct_from_spec_file (e_table, etm, ete, spec_fn, state_fn);
 		
-	return (GtkWidget *) e_table;
+	return GTK_WIDGET (e_table);
 }
 
+#if 0
 static xmlNode *
 et_build_column_spec (ETable *e_table)
 {
@@ -901,6 +1117,24 @@ e_table_get_specification (ETable *e_table)
 	return buffer;
 }
 
+int
+e_table_set_specification (ETable *e_table, const char *spec)
+{
+	xmlDoc *xmlSpec;
+	int ret;
+
+	g_return_val_if_fail(e_table != NULL, -1);
+	g_return_val_if_fail(E_IS_TABLE(e_table), -1);
+	g_return_val_if_fail(spec != NULL, -1);
+
+	/* doesn't work yet, sigh */
+	xmlSpec = xmlParseMemory ((char *)spec, strlen(spec));
+	ret = et_real_set_specification(e_table, xmlSpec);
+	xmlFreeDoc (xmlSpec);
+
+	return ret;
+}
+
 void
 e_table_save_specification (ETable *e_table, gchar *filename)
 {
@@ -914,6 +1148,25 @@ e_table_save_specification (ETable *e_table, gchar *filename)
 	xmlFreeDoc (doc);
 }
 
+int
+e_table_load_specification (ETable *e_table, gchar *filename)
+{
+	xmlDoc *xmlSpec;
+	int ret;
+
+	g_return_val_if_fail(e_table != NULL, -1);
+	g_return_val_if_fail(E_IS_TABLE(e_table), -1);
+	g_return_val_if_fail(filename != NULL, -1);
+
+	/* doesn't work yet, yay */
+	xmlSpec = xmlParseFile (filename);
+	ret = et_real_set_specification(e_table, xmlSpec);
+	xmlFreeDoc (xmlSpec);
+
+	return ret;
+}
+#endif
+
 void
 e_table_set_cursor_row (ETable *e_table, int row)
 {
@@ -985,17 +1238,9 @@ et_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	ETable *etable = E_TABLE (o);
 
 	switch (arg_id){
-	case ARG_TABLE_DRAW_GRID:
-		GTK_VALUE_BOOL (*arg) = etable->draw_grid;
-		break;
-
 	case ARG_TABLE_DRAW_FOCUS:
 		GTK_VALUE_BOOL (*arg) = etable->draw_focus;
 		break;
-
-	case ARG_CLICK_TO_ADD_MESSAGE:
-		GTK_VALUE_STRING (*arg) = g_strdup (etable->click_to_add_message);
-		break;
 	}
 }
 
@@ -1019,15 +1264,6 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 		}
 		break;
 		
-	case ARG_TABLE_DRAW_GRID:
-		etable->draw_grid = GTK_VALUE_BOOL (*arg);
-		if (etable->group) {
-			gnome_canvas_item_set (GNOME_CANVAS_ITEM(etable->group),
-					       "drawgrid", GTK_VALUE_BOOL (*arg),
-					       NULL);
-		}
-		break;
-
 	case ARG_TABLE_DRAW_FOCUS:
 		etable->draw_focus = GTK_VALUE_BOOL (*arg);
 		if (etable->group) {
@@ -1036,24 +1272,6 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 					NULL);
 		}
 		break;
-
-	case ARG_CURSOR_MODE:
-		etable->cursor_mode = GTK_VALUE_INT (*arg);
-		if (etable->group) {
-			gnome_canvas_item_set (GNOME_CANVAS_ITEM(etable->group),
-					"cursor_mode", GTK_VALUE_INT (*arg),
-					NULL);
-		}
-		break;
-	case ARG_CLICK_TO_ADD_MESSAGE:
-		if (etable->click_to_add_message)
-			g_free(etable->click_to_add_message);
-		etable->click_to_add_message = g_strdup(GTK_VALUE_STRING (*arg));
-		if (etable->click_to_add)
-			gnome_canvas_item_set(etable->click_to_add,
-					      "message", etable->click_to_add_message,
-					      NULL);
-		break;
 	}
 }
 	
@@ -1838,16 +2056,10 @@ e_table_class_init (GtkObjectClass *object_class)
 				gtk_marshal_NONE__POINTER_POINTER,
 				GTK_TYPE_NONE, 2, GTK_TYPE_ADJUSTMENT, GTK_TYPE_ADJUSTMENT);
 	
-	gtk_object_add_arg_type ("ETable::drawgrid", GTK_TYPE_BOOL,
-				 GTK_ARG_READWRITE, ARG_TABLE_DRAW_GRID);
 	gtk_object_add_arg_type ("ETable::drawfocus", GTK_TYPE_BOOL,
 				 GTK_ARG_READWRITE, ARG_TABLE_DRAW_FOCUS);
-	gtk_object_add_arg_type ("ETable::cursor_mode", GTK_TYPE_INT,
-				 GTK_ARG_WRITABLE, ARG_CURSOR_MODE);
 	gtk_object_add_arg_type ("ETable::length_threshold", GTK_TYPE_INT,
 				 GTK_ARG_WRITABLE, ARG_LENGTH_THRESHOLD);
-	gtk_object_add_arg_type ("ETable::click_to_add_message", GTK_TYPE_STRING,
-				 GTK_ARG_READWRITE, ARG_CLICK_TO_ADD_MESSAGE);
 }
 
 E_MAKE_TYPE(e_table, "ETable", ETable, e_table_class_init, e_table_init, PARENT_TYPE);
